import greenfoot.Actor;
import greenfoot.Greenfoot;
import greenfoot.GreenfootImage;
import greenfoot.MouseInfo;

import java.util.List;

/**
 * Representation of a piece of waste.
 * 
 * @author Dietrich Boles (University of Oldenburg, Germany)
 * @version 01.12.2009
 */
public abstract class Waste extends Actor {

	final static int MAX_DRAG_TIME = 80;
	final static int SPEED = 3;

	protected boolean dragged;
	protected boolean atTheBottom;
	protected int type;
	protected int time;

	public Waste(int type, int numberOfImages) {
		this.type = type;
		this.dragged = false;
		this.time = 0;
		int number = Greenfoot.getRandomNumber(numberOfImages);
		this.setImage(new GreenfootImage("waste-" + type + "-" + number
				+ ".gif"));
	}

	public abstract Waste createWaste();

	public void act() {
		if (this.atTheBottom) {
			// finished
			return;
		}

		if (this.dragged) {
			this.time++;
		}

		if (this.time >= Waste.MAX_DRAG_TIME) {
			((WasteWorld) this.getWorld()).setToBottom();
			this.atTheBottom = true;
			this.dragged = false;
			this.time = 0;
		} else if (Greenfoot.mousePressed(this) || Greenfoot.mouseDragged(this)) {
			this.dragged = true;
			MouseInfo mouse = Greenfoot.getMouseInfo();
			this.setLocation(mouse.getX(), mouse.getY());

			((WasteWorld) this.getWorld()).checkWasteIntersection();
		} else if (Greenfoot.mouseDragEnded(this)) {
			this.dragged = false;
			((WasteWorld) this.getWorld()).checkDustbinIntersection();
		} else if (!this.dragged) {
			// fall down one row
			this.setLocation(this.getX(), this.getY() + Waste.SPEED);
			this.setRotation(this.getRotation() + 3);
			if (((WasteWorld) this.getWorld()).checkWasteAtTheBottom()) {
				this.atTheBottom = true;
			}
		}
	}

	public int getType() {
		return this.type;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see greenfoot.Actor#getIntersectingObjects(java.lang.Class)
	 */
	@Override
	public List getIntersectingObjects(Class cls) {
		// declaring the method as public
		return super.getIntersectingObjects(cls);
	}
}
